"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LinkedListNode = void 0;
const assertState_1 = __importDefault(require("./assertState"));
var NodeType;
(function (NodeType) {
    NodeType[NodeType["DummyHead"] = 0] = "DummyHead";
    NodeType[NodeType["DummyTail"] = 1] = "DummyTail";
    NodeType[NodeType["List"] = 2] = "List";
})(NodeType || (NodeType = {}));
class ValueNode {
    value;
    detached = false;
    prev;
    next;
    type = NodeType.List;
    constructor(prev, next, value) {
        this.value = value;
        [prev.next, this.prev] = [this, prev];
        [next.prev, this.next] = [this, next];
    }
    detach() {
        (0, assertState_1.default)(!this.detached);
        this.detached = true;
        this.prev.next = this.next;
        this.next.prev = this.prev;
    }
    getPrev() {
        (0, assertState_1.default)(!this.detached);
        const n = this.prev;
        return n.type == NodeType.List ? n : undefined;
    }
    getNext() {
        (0, assertState_1.default)(!this.detached);
        const n = this.next;
        return n.type == NodeType.List ? n : undefined;
    }
    getValue() {
        return this.value;
    }
    setValue(v) {
        this.value = v;
    }
}
exports.LinkedListNode = ValueNode;
class LinkedList {
    // For simplicity, use dummy nodes.
    head;
    tail;
    constructor() {
        this.head = { type: NodeType.DummyHead };
        this.tail = { type: NodeType.DummyTail };
        this.head.next = this.tail;
        this.tail.prev = this.head;
    }
    append(val) {
        const n = new ValueNode(this.tail.prev, this.tail, val);
        return n;
    }
    prepend(val) {
        const n = new ValueNode(this.head, this.head.next, val);
        return n;
    }
    getHead() {
        const n = this.head.next;
        return n.type == NodeType.List ? n : undefined;
    }
    getTail() {
        const n = this.tail.prev;
        return n.type == NodeType.List ? n : undefined;
    }
}
exports.default = LinkedList;
